﻿// #define DrawBox

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Drawing;

namespace howto_draw_equations
{
    // Draw a matrix of Equations.
    class MatrixEquation : Equation
    {
        // The items to draw.
        private int NumRows, NumCols;
        private Equation[,] Items;

        // Space to add between rows and columns;
        private const float RowSpace = 4;
        private const float ColSpace = 4;

        // True if we should make rows/columns have the same sizes.
        private bool UniformRows, UniformCols;

        // Initialize the items.
        public MatrixEquation(int num_rows, int num_cols, bool uniform_rows, bool uniform_cols, params Equation[] items)
        {
            NumRows = num_rows;
            NumCols = num_cols;
            UniformRows = uniform_rows;
            UniformCols= uniform_cols;

            Items = new Equation[num_rows, num_cols];
            for (int i = 0, row = 0; row < NumRows; row++)
            {
                for (int col = 0; col < NumCols; col++, i++)
                {
                    if (i >= items.Length) break;
                    Items[row, col] = items[i];
                }
                if (i >= items.Length) break;
            }
        }

        // Set font sizes for sub-equations.
        public override void SetFontSizes(float font_size)
        {
            FontSize = font_size;
            for (int i = 0, row = 0; row < NumRows; row++)
            {
                for (int col = 0; col < NumCols; col++, i++)
                {
                    Items[row, col].SetFontSizes(font_size);
                }
            }
        }

        // Return the equation's size.
        public override SizeF GetSize(Graphics gr, Font font)
        {
            // Get the row and column sizes.
            float[] row_heights, col_widths;
            MeasureRowsAndColumns(gr, font, out row_heights, out col_widths);

            // Add them up.
            float height = row_heights.Sum() + RowSpace * (NumRows - 1);
            float width = col_widths.Sum() + ColSpace * (NumCols - 1);

            return new SizeF(width, height);
        }

        // Draw the equation.
        public override void Draw(Graphics gr, Font font, Pen pen, Brush brush, float x, float y)
        {
            // Get the row and column sizes.
            float[] row_heights, col_widths;
            MeasureRowsAndColumns(gr, font, out row_heights, out col_widths);

#if DrawBox
            float height = row_heights.Sum() + (row_heights.Length - 1) * RowSpace;
            float width = col_widths.Sum() + (col_widths.Length - 1) * ColSpace;
            using (Pen dashed_pen = new Pen(Color.Blue, 1))
            {
                dashed_pen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dash;
                gr.DrawRectangle(dashed_pen, x, y, width, height);
            }
#endif

            // Draw the items.
            float row_y = y;
            for (int row = 0; row < NumRows; row++)
            {
                float col_x = x;
                for (int col = 0; col < NumCols; col++)
                {
                    if (Items[row, col] != null)
                    {
                        // Get the item's size.
                        SizeF item_size = Items[row, col].GetSize(gr, font);

                        // Draw the item.
                        float item_x = col_x + (col_widths[col] - item_size.Width) / 2;
                        float item_y = row_y + (row_heights[row] - item_size.Height) / 2;
                        Items[row, col].Draw(gr, font, pen, brush, item_x, item_y);
                    }

                    // Move to the next column.
                    col_x += col_widths[col] + ColSpace;
                }

                // Move to the next row.
                row_y += row_heights[row] + RowSpace;
            }
        }

        // Measure the row and column sizes.
        private void MeasureRowsAndColumns(Graphics gr, Font font, out float[] row_heights, out float[] col_widths)
        {
            // Make room for the values.
            row_heights = new float[NumRows];
            col_widths = new float[NumCols];

            // Get the largest row heights and column widths.
            for (int row = 0; row < NumRows; row++)
            {
                for (int col = 0; col < NumCols; col++)
                {
                    if (Items[row, col] != null)
                    {
                        SizeF item_size = Items[row, col].GetSize(gr, font);
                        if (row_heights[row] < item_size.Height) row_heights[row] = item_size.Height;
                        if (col_widths[col] < item_size.Width) col_widths[col] = item_size.Width;
                    }
                }
            }

            // See if we want uniform row heights.
            if (UniformRows)
            {
                // Get the maximum row height.
                float max_height = row_heights.Max();

                // Set all rows to this height.
                for (int row = 0; row < NumRows; row++) row_heights[row] = max_height;
            }

            // See if we want uniform column widths.
            if (UniformCols)
            {
                // Get the maximum col width.
                float max_width = col_widths.Max();

                // Set all cols to this width.
                for (int col = 0; col < NumCols; col++) col_widths[col] = max_width;
            }
        }
    }
}
